;#####################################################################
;#                                                                   #
;#       3D oldschool routs                                          #
;#                                                                   #
;#       by Jace / ST Knights                                        #
;#                                                                   #
;#                                                                   #
;#####################################################################


Structure p2D
  x.f
  y.f
EndStructure
Structure p3D Extends p2D
  z.f
EndStructure

Structure object
  nbPoints.w      ;number of points
  nbLignes.w      ;number of lines
  nbFaces.w       ;number of faces
  *ptPoints.p3D   ;pointer to array of original coords (contructor)
  *ptPointsT.p3D  ;pointer to array 3d rotated  and translated
  *ptPointsP.p2D  ;pointer to array of 2D projected
  *ptLignes       ;pointer to array of type lines
  *ptFaces        ;pointer to array of type faces
  type.w           ;type of object (filled, wireframe, ...)
  angX.f
  angY.f
  angZ.f
  posX.f
  posY.f
  posZ.f
EndStructure

;line struct (coord)
Structure droite
  x1.w
  y1.w
  x2.w
  y2.w
EndStructure

Structure ligne
  p1.w
  p2.w
EndStructure

Structure pQuad
  p1.w
  p2.w
  p3.w
  p4.w
  couleur.l
  normal.b
  type.w
EndStructure

Structure Matrix
  m00.f
  m01.f
  m02.f
  m10.f
  m11.f
  m12.f
  m20.f
  m21.f
  m22.f
EndStructure

Structure pipeLine
  zFace.l
  p1.p2D
  p2.p2D
  p3.p2D
  p4.p2D
  coul.l  
EndStructure


Procedure CalcMatrix(*m.Matrix,tilt.f,turn.f,roll.f,scale.f=1.0)

  cr.f = Cos(tilt)
  sr.f = Sin(tilt)
  ct.f = Cos(turn)
  st.f = Sin(turn)
  cp.f = Cos(roll)
  sp.f = Sin(roll)
  stsr.f = st * sr
  stcr.f = st * cr
  
  *m\m00 = (cp * ct) *scale
  *m\m01 = (sp * ct) *scale
  *m\m02 = (-st) *scale
  
  *m\m10 = (cp * stsr - sp * cr) *scale
  *m\m11 = (sp * stsr + cp * cr) *scale
  *m\m12 = (ct * sr) *scale
  
  *m\m20 = (stcr * cp + sp * sr) *scale
  *m\m21 = (stcr * sp - cp * sr) *scale
  *m\m22 = (ct * cr)*scale

EndProcedure

Procedure visible(*p1.p3D,*p2.p3D,*p3.p3D)
;   ( yba*zca - zba*yca, zba*xca - xba*zca, xba*yca - yba*xca)
;   Si A = (xa,ya,za) et B = (xb,yb,zb), alors AB = (xb-xa, yb-ya, zb-za )
; 
;   Si AB = (xba,yba,zba) et AC = (xca, yca, zca), alors 
;   Protected n.p3D
; 
;   n\x = (*p1\y-*p2\y)*(*p3\z-*p2\z) - (*p1\z-*p2\z)*(*p3\y-*p2\y)
;   n\y = (*p1\z-*p2\z)*(*p3\x-*p2\x) - (*p1\x-*p2\x)*(*p3\z-*p2\z)
;   n\z = (*p1\x-*p2\x)*(*p3\y-*p2\y) - (*p1\y-*p2\y)*(*p3\x-*p2\x)
;   Debug StrF(n\x+","+StrF(n\y)+","+StrF(n\z)

  ;Debug StrF(vecteur)
  
  ProcedureReturn (*p1\x-*p2\x)*(*p3\y-*p2\y) - (*p1\y-*p2\y)*(*p3\x-*p2\x)
EndProcedure

    
Procedure.f incAngle(ang.f,val.f)
  ang + val : If ang>2*#PI : ang - 2*#PI : EndIf : ProcedureReturn ang
EndProcedure

Procedure.f decAngle(*ang.f,val.f)
  ang + val  :If ang<0 : ang + 2*#PI : EndIf : ProcedureReturn ang 
EndProcedure

Procedure RotateObject(*monobjet.object,*matrice.Matrix,tX.f,tY.f,tZ.f)
  ;move and rotate and object in 3D
  *ptP.p3D = *monobjet\ptPoints
  *ptT.p3D = *monobjet\ptPointsT
  
  For p = 0 To *monobjet\nbPoints-1
    
    *ptT\x = (*ptP\x+tX) * *matrice\m00 + (*ptP\y+tY) * *matrice\m10 + (*ptP\z+tZ) * *matrice\m20
    *ptT\y = (*ptP\x+tX) * *matrice\m01 + (*ptP\y+tY) * *matrice\m11 + (*ptP\z+tZ) * *matrice\m21
    *ptT\z = (*ptP\x+tX) * *matrice\m02 + (*ptP\y+tY) * *matrice\m12 + (*ptP\z+tZ) * *matrice\m22
    ;Debug StrF(*ptT\x)+","+StrF(*ptT\y)+","+StrF(*ptT\z)
    *ptP + SizeOf(p3D)
    *ptT + SizeOf(p3D)
    
  Next p
EndProcedure

Procedure translateObject(*objet.object,Dx.w,Dy.w,Dz.w,scale=1.0)
  ;translate an 3D object in 2D with pre-moving x,y,z
  Protected *ptT.p3D = *objet\ptPointsT
  Protected *ptS.p2D = *objet\ptPointsP
  
  For p = 0 To *objet\nbPoints-1
    
    q.f = 1 - ((*ptT\z*scale)+Dz)/1000
    If q.f=0
      *ptS\x = ((*ptT\x*scale)+Dx)/0.001 
      *ptS\y = -((*ptT\y*scale)+Dy)/0.001 
    Else
      *ptS\x = ((*ptT\x*scale)+Dx)/q 
      *ptS\y = -((*ptT\y*scale)+Dy)/q 
    EndIf
  
    *ptT + SizeOf(p3D)
    *ptS + SizeOf(p2D)
  Next p
EndProcedure

Procedure makeDrawPipeline(Array listeObjets.object(1),nbObjects.w,posx.w,posy.w,modeAff=0) ;,*listeFace.pQuad)
  
  Protected Dim monPipe.pipeLine(0)
  Protected nbQuadPipe.w = 0
  
  For p=0 To nbObjects-1
    
    *ptF.pQuad = listeObjets(p)\ptFaces
    ReDim monPipe(nbQuadPipe + listeObjets(p)\nbFaces)      ;dim en prvision
    nbEntered = 0
    For n=1 To listeObjets(p)\nbFaces
      
      *p4.p3D = listeObjets(p)\ptPointsT + *ptF\p4*SizeOf(p3D)
      *p3.p3D = listeObjets(p)\ptPointsT + *ptF\p3*SizeOf(p3D)
      *p2.p3D = listeObjets(p)\ptPointsT + *ptF\p2*SizeOf(p3D)
      *p1.p3D = listeObjets(p)\ptPointsT + *ptF\p1*SizeOf(p3D)
      vect.F = visible(*p3,*p2,*p1) 
      If vect > 0
        monPipe(nbQuadPipe + nbEntered)\zFace = (*p4\z + *p3\z + *p2\z + *p1\z)/0.25
        *ptP.p2D = listeObjets(p)\ptPointsP + *ptF\p1*SizeOf(p2D)
        CopyStructure(*ptP,@monPipe(nbQuadPipe + nbEntered)\p1,p2D)
        ;monPipe(nbQuadPipe + nbEntered)\p1\x = *ptP\x
        ;monPipe(nbQuadPipe + nbEntered)\p1\y = *ptP\y
        *ptP.p2D = listeObjets(p)\ptPointsP + *ptF\p2*SizeOf(p2D)
        CopyStructure(*ptP,@monPipe(nbQuadPipe + nbEntered)\p2,p2D)
        ;monPipe(nbQuadPipe + nbEntered)\p2\x = *ptP\x
        ;monPipe(nbQuadPipe + nbEntered)\p2\y = *ptP\y
        *ptP.p2D = listeObjets(p)\ptPointsP + *ptF\p3*SizeOf(p2D)
        CopyStructure(*ptP,@monPipe(nbQuadPipe + nbEntered)\p3,p2D)
        ;monPipe(nbQuadPipe + nbEntered)\p3\x = *ptP\x
        ;monPipe(nbQuadPipe + nbEntered)\p3\y = *ptP\y
        *ptP.p2D = listeObjets(p)\ptPointsP + *ptF\p4*SizeOf(p2D)
        CopyStructure(*ptP,@monPipe(nbQuadPipe + nbEntered)\p4,p2D)
        ;monPipe(nbQuadPipe + nbEntered)\p4\x = *ptP\x
        ;monPipe(nbQuadPipe + nbEntered)\p4\y = *ptP\y
        R.w = (*ptF\couleur) & $ff
        R * (vect/10000) 
        If R>255
          R = 255
        EndIf
        V.w = (*ptF\couleur)>>8 & $ff
        V * (vect/10000)
        If V>255
          V = 255
        EndIf
        B.w = (*ptF\couleur)>>16 & $ff
        b * (vect/10000)
        If B>255
          B = 255
        EndIf
        
        monPipe(nbQuadPipe + nbEntered)\coul = RGB(R,V,B) ;*ptF\couleur
        nbEntered + 1
      EndIf
      *ptF + SizeOf(pQuad)        
    Next n
    nbQuadPipe + nbEntered
    ReDim monPipe(nbQuadPipe)                           ;redim exact

  Next p
  
  SortStructuredArray(monPipe(), #PB_Sort_Ascending,OffsetOf(pipeLine\zFace),#PB_Sort_Long,0,ArraySize(monPipe())-1)
  
  
  ;Define pd3d.IDirect3DDevice9
  ;Start3D()
  ;EnableASM
  ;!extrn _PB_Screen_Direct3DDevice
  ;!MOV dword EAX, [_PB_Screen_Direct3DDevice]
  ;!MOV dword [v_pd3d],EAX
  ;DisableASM
  ;pd3d\SetRenderState(22,1)
  

  For p=0 To nbQuadPipe-1
    Select modeAff
      Case 0
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,monPipe(p)\coul)
        StopDrawing()
        TransformSprite3D(100,monPipe(p)\p4\x,monPipe(p)\p4\y,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        ;Debug StrF(monPipe(p)\p4\x)+","+StrF(monPipe(p)\p4\y) ;,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        DisplaySprite3D(100,posx,posy)
      Case 1
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,monPipe(p)\coul)
        StopDrawing()
        TransformSprite3D(100,monPipe(p)\p4\x,monPipe(p)\p4\y,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        ;Debug StrF(monPipe(p)\p4\x)+","+StrF(monPipe(p)\p4\y) ;,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        DisplaySprite3D(100,posx,posy)
        StartDrawing(ScreenOutput())
        *ptEcran = DrawingBuffer()
        lgEcran = DrawingBufferPitch()
        StopDrawing()
        LigneRetro(monPipe(p)\p4\x+posx,monPipe(p)\p4\y+posy,monPipe(p)\p3\x+posx,monPipe(p)\p3\y+posy,-1)
        LigneRetro(monPipe(p)\p3\x+posx,monPipe(p)\p3\y+posy,monPipe(p)\p2\x+posx,monPipe(p)\p2\y+posy,-1) 
        LigneRetro(monPipe(p)\p2\x+posx,monPipe(p)\p2\y+posy,monPipe(p)\p1\x+posx,monPipe(p)\p1\y+posy,-1)
        LigneRetro(monPipe(p)\p1\x+posx,monPipe(p)\p1\y+posy,monPipe(p)\p4\x+posx,monPipe(p)\p4\y+posy,-1)
      Case 2
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,monPipe(p)\coul)
        StopDrawing()
        TransformSprite3D(100,monPipe(p)\p4\x,monPipe(p)\p4\y,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        ;Debug StrF(monPipe(p)\p4\x)+","+StrF(monPipe(p)\p4\y) ;,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        DisplaySprite3D(100,posx,posy,192)
        StartDrawing(ScreenOutput())
        *ptEcran = DrawingBuffer()
        lgEcran = DrawingBufferPitch()
        StopDrawing()
        LigneRetro(monPipe(p)\p4\x+posx,monPipe(p)\p4\y+posy,monPipe(p)\p3\x+posx,monPipe(p)\p3\y+posy,-1)
        LigneRetro(monPipe(p)\p3\x+posx,monPipe(p)\p3\y+posy,monPipe(p)\p2\x+posx,monPipe(p)\p2\y+posy,-1) 
        LigneRetro(monPipe(p)\p2\x+posx,monPipe(p)\p2\y+posy,monPipe(p)\p1\x+posx,monPipe(p)\p1\y+posy,-1)
        LigneRetro(monPipe(p)\p1\x+posx,monPipe(p)\p1\y+posy,monPipe(p)\p4\x+posx,monPipe(p)\p4\y+posy,-1)

      Case 3
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,monPipe(p)\coul)
        StopDrawing()
        TransformSprite3D(100,monPipe(p)\p4\x,monPipe(p)\p4\y,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        ;Debug StrF(monPipe(p)\p4\x)+","+StrF(monPipe(p)\p4\y) ;,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        DisplaySprite3D(100,posx,posy)
        Dim pDepart.p2D(6)
        Dim pArrivee.p2D(6)
        deltax.f = ((monPipe(p)\p4\x+posx)-(monPipe(p)\p3\x+posx))/5
        deltay.f = ((monPipe(p)\p4\y+posy)-(monPipe(p)\p3\y+posy))/5
        xpt.f = monPipe(p)\p3\x+posx
        ypt.f = monPipe(p)\p3\y+posy
        StartDrawing(ScreenOutput())       
        For i=0 To 5
          pDepart(i)\x = xpt
          pDepart(i)\y = ypt
          xpt + deltax
          ypt + deltay
        Next

        deltax = ((monPipe(p)\p1\x+posx)-(monPipe(p)\p2\x+posx))/5
        deltay = ((monPipe(p)\p1\y+posy)-(monPipe(p)\p2\y+posy))/5
        xpt = monPipe(p)\p2\x+posx
        ypt = monPipe(p)\p2\y+posy
        
        For i=0 To 5
          pArrivee(i)\x = xpt
          pArrivee(i)\y = ypt
          ;Plot(xpt,ypt,RGB(255,255,255))
          xpt + deltax
          ypt + deltay
        Next

        For i=0 To 5
          deltax = ((pDepart(i)\x)-(pArrivee(i)\x))/5
          deltay = ((pDepart(i)\y)-(pArrivee(i)\y))/5
          xpt = pArrivee(i)\x
          ypt = pArrivee(i)\y
          For j=0 To 5
            Plot(xpt,ypt,RGB(255,255,255))
            xpt + deltax
            ypt + deltay
          Next j       
        Next i  
        StopDrawing()
       Case 4
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,monPipe(p)\coul)
        StopDrawing()
        TransformSprite3D(100,monPipe(p)\p4\x,monPipe(p)\p4\y,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        ;Debug StrF(monPipe(p)\p4\x)+","+StrF(monPipe(p)\p4\y) ;,monPipe(p)\p3\x,monPipe(p)\p3\y,monPipe(p)\p2\x,monPipe(p)\p2\y,monPipe(p)\p1\x,monPipe(p)\p1\y)
        DisplaySprite3D(100,posx,posy,192)
        Dim pDepart.p2D(6)
        Dim pArrivee.p2D(6)
        deltax.f = ((monPipe(p)\p4\x+posx)-(monPipe(p)\p3\x+posx))/5
        deltay.f = ((monPipe(p)\p4\y+posy)-(monPipe(p)\p3\y+posy))/5
        xpt.f = monPipe(p)\p3\x+posx
        ypt.f = monPipe(p)\p3\y+posy
        StartDrawing(ScreenOutput())       
        For i=0 To 5
          pDepart(i)\x = xpt
          pDepart(i)\y = ypt
          xpt + deltax
          ypt + deltay
        Next

        deltax = ((monPipe(p)\p1\x+posx)-(monPipe(p)\p2\x+posx))/5
        deltay = ((monPipe(p)\p1\y+posy)-(monPipe(p)\p2\y+posy))/5
        xpt = monPipe(p)\p2\x+posx
        ypt = monPipe(p)\p2\y+posy
        
        For i=0 To 5
          pArrivee(i)\x = xpt
          pArrivee(i)\y = ypt
          ;Plot(xpt,ypt,RGB(255,255,255))
          xpt + deltax
          ypt + deltay
        Next

        For i=0 To 5
          deltax = ((pDepart(i)\x)-(pArrivee(i)\x))/5
          deltay = ((pDepart(i)\y)-(pArrivee(i)\y))/5
          xpt = pArrivee(i)\x
          ypt = pArrivee(i)\y
          For j=0 To 5
            Plot(xpt,ypt,RGB(255,255,255))
            xpt + deltax
            ypt + deltay
          Next j       
        Next i  
        StopDrawing()
    Case 5
        Dim pDepart.p2D(6)
        Dim pArrivee.p2D(6)
        deltax.f = ((monPipe(p)\p4\x+posx)-(monPipe(p)\p3\x+posx))/5
        deltay.f = ((monPipe(p)\p4\y+posy)-(monPipe(p)\p3\y+posy))/5
        xpt.f = monPipe(p)\p3\x+posx
        ypt.f = monPipe(p)\p3\y+posy
        StartDrawing(ScreenOutput())       
        For i=0 To 5
          pDepart(i)\x = xpt
          pDepart(i)\y = ypt
          xpt + deltax
          ypt + deltay
        Next

        deltax = ((monPipe(p)\p1\x+posx)-(monPipe(p)\p2\x+posx))/5
        deltay = ((monPipe(p)\p1\y+posy)-(monPipe(p)\p2\y+posy))/5
        xpt = monPipe(p)\p2\x+posx
        ypt = monPipe(p)\p2\y+posy
        
        For i=0 To 5
          pArrivee(i)\x = xpt
          pArrivee(i)\y = ypt
          ;Plot(xpt,ypt,RGB(255,255,255))
          xpt + deltax
          ypt + deltay
        Next

        For i=0 To 5
          deltax = ((pDepart(i)\x)-(pArrivee(i)\x))/5
          deltay = ((pDepart(i)\y)-(pArrivee(i)\y))/5
          xpt = pArrivee(i)\x
          ypt = pArrivee(i)\y
          For j=0 To 5
            Plot(xpt,ypt,RGB(255,255,255))
            xpt + deltax
            ypt + deltay
          Next j       
        Next i  
        StopDrawing()

    EndSelect

  Next
  ;Stop3D()

  
EndProcedure

Procedure drawSimpleObject(*objet.object,posx.w,posy.w,type)
  Protected Dim sommets.p2D(*objet\nbPoints)
  ;CopyMemory(*objet\ptPointsP,sommets(),SizeOf(p2D)*objet\nbPoints)
  *ptP.p2D = *objet\ptPointsP
  For p = 0 To *objet\nbPoints-1
    sommets(p)\x = *ptP\x
    sommets(p)\y = *ptP\y
    *ptP + SizeOf(p2D)
  Next p
  ;Debug *objet\type
  Select type
    Case 1      ;Points
      *ptP.p2D = *objet\ptPointsP
      StartDrawing(ScreenOutput())
      ;Debug *objet\nbPoints
      For p=0 To *objet\nbPoints-1
        x.w = *ptP\x+posx
        y.w = *ptP\y+posy
        If x>0 And x<800 And y>0 And y<600
          ;Plot(x,y,fastRGB(255,255,255))
          Box(x,y,16,16,RGB(255,255,255))
          ;Debug StrF(sommets(p)\x)+","+StrF(sommets(p)\y)
        EndIf
        *ptP + SizeOf(p2D)
      Next p
      StopDrawing()
      
    Case 2:     ;filled
      ; 3d remplie
      
      ;Define pd3d.IDirect3DDevice9
      Start3D()
      ;EnableASM
      ;!extrn _PB_Screen_Direct3DDevice
      ;!MOV dword EAX, [_PB_Screen_Direct3DDevice]
      ;!MOV dword [v_pd3d],EAX
      ;DisableASM
      ;pd3d\SetRenderState(22,1)

      *ptF.pQuad = *objet\ptFaces
      For p = 0 To *objet\nbFaces-1
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,*ptF\couleur)
        StopDrawing()
;        TransformSprite3D(100,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y)
        TransformSprite3D(100,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y)
        DisplaySprite3D(100,posx,posy)
        *ptF + SizeOf(pQuad)      
      Next p
      Stop3D()

    Case 3:
      ;wireframe
      StartDrawing(ScreenOutput())
      *ptL.ligne = *objet\ptLignes
      For p = 0 To *objet\nbLignes-1
        LineXY(sommets(*ptL\p1)\x+posx, sommets(*ptL\p1)\y+posy, sommets(*ptL\p2)\x+posx, sommets(*ptL\p2)\y+posy, RGB(255,0,0))
        *ptL + SizeOf(ligne)  
      Next p
      StopDrawing()
      
    Case 4:
      ;face visibles
;      Debug "top"
      Start3D() 
       *ptF.pQuad = *objet\ptFaces
       For p = 0 To *objet\nbFaces-1
          If visible(*objet\ptPointsT + *ptF\p3*SizeOf(p3D),*objet\ptPointsT + *ptF\p2*SizeOf(p3D),*objet\ptPointsT + *ptF\p1*SizeOf(p3D))>=0
          StartDrawing(SpriteOutput(100))
          Box(0,0,32,32,*ptF\couleur)
          StopDrawing()
          TransformSprite3D(100,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y)
          DisplaySprite3D(100,posx,posy)
        EndIf
        *ptF + SizeOf(pQuad)      
      Next p
      Stop3D()

  EndSelect
  
EndProcedure

Procedure drawObject2(*objet.object,Dx.w,Dy.w,Dz.w,type)
  
  Protected *ptT.p3D = *objet\ptPointsT
  Protected Dim sommets.p2D(*objet\nbPoints)
  

  For p = 0 To *objet\nbPoints-1
    
    q.f = 1 - (*ptT\z+Dz)/500
    If q.f=0
      sommets(p)\x = (*ptT\x+Dx)/0.001 
      sommets(p)\y = -(*ptT\y+Dy)/0.001 
    Else
      sommets(p)\x = (*ptT\x+Dx)/q 
      sommets(p)\y = -(*ptT\y+Dy)/q 
    EndIf
  
    *ptT + SizeOf(p3D)
    
  Next p
  
  Select type
    Case 1
      StartDrawing(ScreenOutput())
      ;Debug *objet\nbPoints
      For p=0 To *objet\nbPoints-1
        x.w = sommets(p)\x+400
        y.w = sommets(p)\y+300
        If x>0 And x<800 And y>0 And y<600
          ;Plot(x,y,fastRGB(255,255,255))
          Box(x,y,16,16,RGB(255,255,255))
          ;Debug StrF(sommets(p)\x)+","+StrF(sommets(p)\y)
        EndIf
      Next p
      StopDrawing()
      
    Case 2:
      ; 3d remplie
      
      ;Define pd3d.IDirect3DDevice9
      Start3D()
      ;EnableASM
      ;!extrn _PB_Screen_Direct3DDevice
      ;!MOV dword EAX, [_PB_Screen_Direct3DDevice]
      ;!MOV dword [v_pd3d],EAX
      ;DisableASM
      ;pd3d\SetRenderState(22,1)
      
      *ptF.pQuad = *objet\ptFaces
      For p = 0 To *objet\nbFaces-1
        StartDrawing(SpriteOutput(100))
        Box(0,0,32,32,*ptF\couleur)
        StopDrawing()
;        TransformSprite3D(100,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y)
        TransformSprite3D(100,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y)
        DisplaySprite3D(100,400,300)
         *ptF + SizeOf(pQuad)      
      Next p
      Stop3D()

    Case 3:
      ;fil de fer
      StartDrawing(ScreenOutput())
      *ptL.ligne = *objet\ptLignes
      For p = 0 To *objet\nbLignes-1
        LineXY(sommets(*ptL\p1)\x+400, sommets(*ptL\p1)\y+300, sommets(*ptL\p2)\x+400, sommets(*ptL\p2)\y+300, RGB(255,0,0))
        *ptL + SizeOf(ligne)  
      Next p
      StopDrawing()
      
    Case 4:
      ;face visibles
;      Debug "top"
      Start3D() 
       *ptF.pQuad = *objet\ptFaces
       For p = 0 To *objet\nbFaces-1
          If visible(*objet\ptPointsT + *ptF\p3*SizeOf(p3D),*objet\ptPointsT + *ptF\p2*SizeOf(p3D),*objet\ptPointsT + *ptF\p1*SizeOf(p3D))>=0
          StartDrawing(SpriteOutput(100))
          Box(0,0,32,32,*ptF\couleur)
          StopDrawing()
          TransformSprite3D(100,sommets(*ptF\p4)\x,sommets(*ptF\p4)\y,sommets(*ptF\p3)\x,sommets(*ptF\p3)\y,sommets(*ptF\p2)\x,sommets(*ptF\p2)\y,sommets(*ptF\p1)\x,sommets(*ptF\p1)\y)
          DisplaySprite3D(100,400,300)
        EndIf
        *ptF + SizeOf(pQuad)      
      Next p
      Stop3D()

  EndSelect
  
EndProcedure


; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 6
; FirstLine = 455
; Folding = +0
; EnableXP